#!/usr/bin/env python3
# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4
# -*- coding: latin-1 -*-
"""
__version__.py
"""

# Copyright 2022 Ball Aerospace & Technologies Corp.
# All Rights Reserved.
#
# This program is free software; you can modify and/or redistribute it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; version 3 with
# attribution addendums as found in the LICENSE.txt

__title__ = "cosmosc2"
__description__ = "Python Support for Ball Aerospace COSMOS v5"
__url__ = "https://github.com/BallAerospace/python-cosmosc2"
__version__ = "1.2.0"
